# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
import datetime as dt
from .embedding_config import EmbeddingConfig
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Passage(UncheckedBaseModel):
    """
    Representation of a passage, which is stored in archival memory.

    Parameters:
        text (str): The text of the passage.
        embedding (List[float]): The embedding of the passage.
        embedding_config (EmbeddingConfig): The embedding configuration used by the passage.
        created_at (datetime): The creation date of the passage.
        user_id (str): The unique identifier of the user associated with the passage.
        agent_id (str): The unique identifier of the agent associated with the passage.
        source_id (str): The data source of the passage.
        file_id (str): The unique identifier of the file associated with the passage.
    """

    created_by_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The id of the user that made this object.
    """

    last_updated_by_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The id of the user that made this object.
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The creation date of the passage.
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The timestamp when the object was last updated.
    """

    is_deleted: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this passage is deleted or not.
    """

    agent_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the agent associated with the passage.
    """

    source_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The data source of the passage.
    """

    file_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the file associated with the passage.
    """

    metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    The metadata of the passage.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The human-friendly ID of the Passage
    """

    text: str = pydantic.Field()
    """
    The text of the passage.
    """

    embedding: typing.Optional[typing.List[float]] = pydantic.Field(default=None)
    """
    The embedding of the passage.
    """

    embedding_config: typing.Optional[EmbeddingConfig] = pydantic.Field(default=None)
    """
    The embedding configuration used by the passage.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from .templates_create_agents_response_agents_item_tool_rules_item_item_children import (
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildren,
)
from .templates_create_agents_response_agents_item_tool_rules_item_item_one import (
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemOne,
)
from .templates_create_agents_response_agents_item_tool_rules_item_item_two import (
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemTwo,
)
from .templates_create_agents_response_agents_item_tool_rules_item_item_child_output_mapping import (
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildOutputMapping,
)
from .templates_create_agents_response_agents_item_tool_rules_item_item_tool_name import (
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemToolName,
)

TemplatesCreateAgentsResponseAgentsItemToolRulesItemItem = typing.Union[
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildren,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemOne,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemTwo,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildOutputMapping,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemToolName,
]



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
from .agents_search_response_agents_item_memory_blocks_item import AgentsSearchResponseAgentsItemMemoryBlocksItem
from .agents_search_response_agents_item_memory_prompt_template import (
    AgentsSearchResponseAgentsItemMemoryPromptTemplate,
)
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class AgentsSearchResponseAgentsItemMemory(UncheckedBaseModel):
    blocks: typing.List[AgentsSearchResponseAgentsItemMemoryBlocksItem]
    prompt_template: typing.Optional[AgentsSearchResponseAgentsItemMemoryPromptTemplate] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigEmbeddingEndpointType = typing.Union[
    typing.Literal[
        "openai",
        "anthropic",
        "bedrock",
        "cohere",
        "google_ai",
        "google_vertex",
        "azure",
        "groq",
        "ollama",
        "webui",
        "webui-legacy",
        "lmstudio",
        "lmstudio-legacy",
        "llamacpp",
        "koboldcpp",
        "vllm",
        "hugging-face",
        "mistral",
        "together",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from .templates_create_agents_response_agents_item_llm_config_put_inner_thoughts_in_kwargs_item import (
    TemplatesCreateAgentsResponseAgentsItemLlmConfigPutInnerThoughtsInKwargsItem,
)

TemplatesCreateAgentsResponseAgentsItemLlmConfigPutInnerThoughtsInKwargs = typing.Union[
    bool,
    typing.Optional[str],
    typing.List[typing.Optional[TemplatesCreateAgentsResponseAgentsItemLlmConfigPutInnerThoughtsInKwargsItem]],
    typing.Optional[typing.Any],
]



# This file was auto-generated by Fern from our API Definition.

import typing
from .templates_create_agents_response_agents_item_tools_item_created_by_id_item import (
    TemplatesCreateAgentsResponseAgentsItemToolsItemCreatedByIdItem,
)

TemplatesCreateAgentsResponseAgentsItemToolsItemCreatedById = typing.Union[
    str,
    typing.Optional[str],
    typing.List[typing.Optional[TemplatesCreateAgentsResponseAgentsItemToolsItemCreatedByIdItem]],
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ...types.conditional_tool_rule import ConditionalToolRule
from ...types.child_tool_rule import ChildToolRule
from ...types.continue_tool_rule import ContinueToolRule
from ...types.terminal_tool_rule import TerminalToolRule
from ...types.init_tool_rule import InitToolRule

UpdateAgentToolRulesItem = typing.Union[
    ConditionalToolRule, ChildToolRule, ContinueToolRule, TerminalToolRule, InitToolRule
]



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
from .agents_search_response_agents_item_tools_item_tool_type import AgentsSearchResponseAgentsItemToolsItemToolType
from .agents_search_response_agents_item_tools_item_description import (
    AgentsSearchResponseAgentsItemToolsItemDescription,
)
from .agents_search_response_agents_item_tools_item_source_type import AgentsSearchResponseAgentsItemToolsItemSourceType
from .agents_search_response_agents_item_tools_item_organization_id import (
    AgentsSearchResponseAgentsItemToolsItemOrganizationId,
)
from .agents_search_response_agents_item_tools_item_name import AgentsSearchResponseAgentsItemToolsItemName
from .agents_search_response_agents_item_tools_item_source_code import AgentsSearchResponseAgentsItemToolsItemSourceCode
from .agents_search_response_agents_item_tools_item_json_schema import AgentsSearchResponseAgentsItemToolsItemJsonSchema
from .agents_search_response_agents_item_tools_item_created_by_id import (
    AgentsSearchResponseAgentsItemToolsItemCreatedById,
)
from .agents_search_response_agents_item_tools_item_last_updated_by_id import (
    AgentsSearchResponseAgentsItemToolsItemLastUpdatedById,
)
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class AgentsSearchResponseAgentsItemToolsItem(UncheckedBaseModel):
    id: typing.Optional[str] = None
    tool_type: typing.Optional[AgentsSearchResponseAgentsItemToolsItemToolType] = None
    description: typing.Optional[AgentsSearchResponseAgentsItemToolsItemDescription] = None
    source_type: typing.Optional[AgentsSearchResponseAgentsItemToolsItemSourceType] = None
    organization_id: typing.Optional[AgentsSearchResponseAgentsItemToolsItemOrganizationId] = None
    name: typing.Optional[AgentsSearchResponseAgentsItemToolsItemName] = None
    tags: typing.Optional[typing.List[str]] = None
    source_code: typing.Optional[AgentsSearchResponseAgentsItemToolsItemSourceCode] = None
    json_schema: typing.Optional[AgentsSearchResponseAgentsItemToolsItemJsonSchema] = None
    return_char_limit: typing.Optional[float] = None
    created_by_id: typing.Optional[AgentsSearchResponseAgentsItemToolsItemCreatedById] = None
    last_updated_by_id: typing.Optional[AgentsSearchResponseAgentsItemToolsItemLastUpdatedById] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

AgentsSearchResponseAgentsItemToolsItemJsonSchemaItem = typing.Union[typing.Optional[typing.Any], typing.Optional[str]]



# This file was auto-generated by Fern from our API Definition.

import typing

TemplatesCreateAgentsResponseAgentsItemAgentType = typing.Union[
    typing.Literal["memgpt_agent", "split_thread_agent", "offline_memory_agent", "chat_only_agent"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigEmbeddingChunkSizeItem = typing.Union[float, typing.Optional[str]]



# This file was auto-generated by Fern from our API Definition.

import typing
from .agents_search_response_agents_item_tool_rules_item_item import AgentsSearchResponseAgentsItemToolRulesItemItem

AgentsSearchResponseAgentsItemToolRulesItem = typing.Union[
    typing.List[AgentsSearchResponseAgentsItemToolRulesItemItem], typing.Optional[str]
]



# This file was auto-generated by Fern from our API Definition.

import typing

NotFoundErrorBodyMessage = typing.Union[
    typing.Literal[
        "Agent provided is a template or not found, you can only migrate deployed agents",
        "Template version provided does not exist",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .openai_types_chat_chat_completion_message_tool_call_function import (
    OpenaiTypesChatChatCompletionMessageToolCallFunction,
)
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class ChatCompletionMessageToolCallInput(UncheckedBaseModel):
    id: str
    function: OpenaiTypesChatChatCompletionMessageToolCallFunction
    type: typing.Literal["function"] = "function"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from .templates_create_agents_response_agents_item_project_id_item import (
    TemplatesCreateAgentsResponseAgentsItemProjectIdItem,
)

TemplatesCreateAgentsResponseAgentsItemProjectId = typing.Union[
    str,
    typing.Optional[str],
    typing.List[typing.Optional[TemplatesCreateAgentsResponseAgentsItemProjectIdItem]],
    typing.Optional[typing.Any],
]



# This file was auto-generated by Fern from our API Definition.

import typing
from .agents_search_response_agents_item_tool_exec_environment_variables_item_created_by_id_item import (
    AgentsSearchResponseAgentsItemToolExecEnvironmentVariablesItemCreatedByIdItem,
)

AgentsSearchResponseAgentsItemToolExecEnvironmentVariablesItemCreatedById = typing.Union[
    str,
    typing.Optional[str],
    typing.List[typing.Optional[AgentsSearchResponseAgentsItemToolExecEnvironmentVariablesItemCreatedByIdItem]],
    typing.Optional[typing.Any],
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.provider import Provider
from ..core.unchecked_base_model import construct_type
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ProvidersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_providers(
        self,
        *,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Provider]:
        """
        Get a list of all custom providers in the database

        Parameters
        ----------
        after : typing.Optional[str]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Provider]
            Successful Response

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            token="YOUR_TOKEN",
        )
        client.providers.list_providers()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/providers/",
            method="GET",
            params={
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Provider],
                    construct_type(
                        type_=typing.List[Provider],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_provider(
        self, *, name: str, api_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> Provider:
        """
        Create a new custom provider

        Parameters
        ----------
        name : str
            The name of the provider.

        api_key : str
            API key used for requests to the provider.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Provider
            Successful Response

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            token="YOUR_TOKEN",
        )
        client.providers.create_provider(
            name="name",
            api_key="api_key",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/providers/",
            method="POST",
            json={
                "name": name,
                "api_key": api_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Provider,
                    construct_type(
                        type_=Provider,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_provider(
        self, *, provider_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[typing.Any]:
        """
        Delete an existing custom provider

        Parameters
        ----------
        provider_id : str
            The provider_id key to be deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            token="YOUR_TOKEN",
        )
        client.providers.delete_provider(
            provider_id="provider_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/providers/",
            method="DELETE",
            params={
                "provider_id": provider_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[typing.Any],
                    construct_type(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def modify_provider(
        self, *, id: str, api_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> Provider:
        """
        Update an existing custom provider

        Parameters
        ----------
        id : str
            The id of the provider to update.

        api_key : str
            API key used for requests to the provider.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Provider
            Successful Response

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            token="YOUR_TOKEN",
        )
        client.providers.modify_provider(
            id="id",
            api_key="api_key",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/providers/",
            method="PATCH",
            json={
                "id": id,
                "api_key": api_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Provider,
                    construct_type(
                        type_=Provider,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncProvidersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_providers(
        self,
        *,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Provider]:
        """
        Get a list of all custom providers in the database

        Parameters
        ----------
        after : typing.Optional[str]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Provider]
            Successful Response

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.providers.list_providers()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/providers/",
            method="GET",
            params={
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Provider],
                    construct_type(
                        type_=typing.List[Provider],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_provider(
        self, *, name: str, api_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> Provider:
        """
        Create a new custom provider

        Parameters
        ----------
        name : str
            The name of the provider.

        api_key : str
            API key used for requests to the provider.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Provider
            Successful Response

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.providers.create_provider(
                name="name",
                api_key="api_key",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/providers/",
            method="POST",
            json={
                "name": name,
                "api_key": api_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Provider,
                    construct_type(
                        type_=Provider,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_provider(
        self, *, provider_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[typing.Any]:
        """
        Delete an existing custom provider

        Parameters
        ----------
        provider_id : str
            The provider_id key to be deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.providers.delete_provider(
                provider_id="provider_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/providers/",
            method="DELETE",
            params={
                "provider_id": provider_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[typing.Any],
                    construct_type(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def modify_provider(
        self, *, id: str, api_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> Provider:
        """
        Update an existing custom provider

        Parameters
        ----------
        id : str
            The id of the provider to update.

        api_key : str
            API key used for requests to the provider.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Provider
            Successful Response

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.providers.modify_provider(
                id="id",
                api_key="api_key",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/providers/",
            method="PATCH",
            json={
                "id": id,
                "api_key": api_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Provider,
                    construct_type(
                        type_=Provider,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .types import (
    TemplatesCreateAgentsResponse,
    TemplatesCreateAgentsResponseAgentsItem,
    TemplatesCreateAgentsResponseAgentsItemAgentType,
    TemplatesCreateAgentsResponseAgentsItemBaseTemplateId,
    TemplatesCreateAgentsResponseAgentsItemBaseTemplateIdItem,
    TemplatesCreateAgentsResponseAgentsItemCreatedAt,
    TemplatesCreateAgentsResponseAgentsItemCreatedAtItem,
    TemplatesCreateAgentsResponseAgentsItemCreatedById,
    TemplatesCreateAgentsResponseAgentsItemCreatedByIdItem,
    TemplatesCreateAgentsResponseAgentsItemDescription,
    TemplatesCreateAgentsResponseAgentsItemDescriptionItem,
    TemplatesCreateAgentsResponseAgentsItemEmbeddingConfig,
    TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigAzureDeployment,
    TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigAzureDeploymentItem,
    TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigAzureEndpoint,
    TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigAzureEndpointItem,
    TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigAzureVersion,
    TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigAzureVersionItem,
    TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigEmbeddingChunkSize,
    TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigEmbeddingChunkSizeItem,
    TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigEmbeddingEndpoint,
    TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigEmbeddingEndpointItem,
    TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigEmbeddingEndpointType,
    TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigHandle,
    TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigHandleItem,
    TemplatesCreateAgentsResponseAgentsItemIdentityIds,
    TemplatesCreateAgentsResponseAgentsItemLastUpdatedById,
    TemplatesCreateAgentsResponseAgentsItemLastUpdatedByIdItem,
    TemplatesCreateAgentsResponseAgentsItemLlmConfig,
    TemplatesCreateAgentsResponseAgentsItemLlmConfigHandle,
    TemplatesCreateAgentsResponseAgentsItemLlmConfigHandleItem,
    TemplatesCreateAgentsResponseAgentsItemLlmConfigMaxTokens,
    TemplatesCreateAgentsResponseAgentsItemLlmConfigMaxTokensItem,
    TemplatesCreateAgentsResponseAgentsItemLlmConfigModelEndpoint,
    TemplatesCreateAgentsResponseAgentsItemLlmConfigModelEndpointItem,
    TemplatesCreateAgentsResponseAgentsItemLlmConfigModelEndpointType,
    TemplatesCreateAgentsResponseAgentsItemLlmConfigModelWrapper,
    TemplatesCreateAgentsResponseAgentsItemLlmConfigModelWrapperItem,
    TemplatesCreateAgentsResponseAgentsItemLlmConfigPutInnerThoughtsInKwargs,
    TemplatesCreateAgentsResponseAgentsItemLlmConfigPutInnerThoughtsInKwargsItem,
    TemplatesCreateAgentsResponseAgentsItemLlmConfigTemperature,
    TemplatesCreateAgentsResponseAgentsItemMemory,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItem,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemCreatedById,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemCreatedByIdItem,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemDescription,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemDescriptionItem,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemId,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemIsTemplate,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemLabel,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemLabelItem,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemLastUpdatedById,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemLastUpdatedByIdItem,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemLimit,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemMetadata,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemMetadataItem,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemName,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemNameItem,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemOrganizationId,
    TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemOrganizationIdItem,
    TemplatesCreateAgentsResponseAgentsItemMemoryPromptTemplate,
    TemplatesCreateAgentsResponseAgentsItemMessageBufferAutoclear,
    TemplatesCreateAgentsResponseAgentsItemMessageIds,
    TemplatesCreateAgentsResponseAgentsItemMessageIdsItem,
    TemplatesCreateAgentsResponseAgentsItemMetadata,
    TemplatesCreateAgentsResponseAgentsItemMetadataItem,
    TemplatesCreateAgentsResponseAgentsItemOrganizationId,
    TemplatesCreateAgentsResponseAgentsItemOrganizationIdItem,
    TemplatesCreateAgentsResponseAgentsItemProjectId,
    TemplatesCreateAgentsResponseAgentsItemProjectIdItem,
    TemplatesCreateAgentsResponseAgentsItemSourcesItem,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemCreatedAt,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemCreatedAtItem,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemCreatedById,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemCreatedByIdItem,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemDescription,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemDescriptionItem,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfig,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigAzureDeployment,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigAzureDeploymentItem,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigAzureEndpoint,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigAzureEndpointItem,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigAzureVersion,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigAzureVersionItem,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigEmbeddingChunkSize,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigEmbeddingChunkSizeItem,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigEmbeddingEndpoint,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigEmbeddingEndpointItem,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigEmbeddingEndpointType,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigHandle,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigHandleItem,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemId,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemLastUpdatedById,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemLastUpdatedByIdItem,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemMetadata,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemMetadataItem,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemOrganizationId,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemOrganizationIdItem,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemUpdatedAt,
    TemplatesCreateAgentsResponseAgentsItemSourcesItemUpdatedAtItem,
    TemplatesCreateAgentsResponseAgentsItemTemplateId,
    TemplatesCreateAgentsResponseAgentsItemTemplateIdItem,
    TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariables,
    TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItem,
    TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemCreatedAt,
    TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemCreatedAtItem,
    TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemCreatedById,
    TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemCreatedByIdItem,
    TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemDescription,
    TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemDescriptionItem,
    TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemId,
    TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemLastUpdatedById,
    TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemLastUpdatedByIdItem,
    TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemOrganizationId,
    TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemOrganizationIdItem,
    TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemUpdatedAt,
    TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemUpdatedAtItem,
    TemplatesCreateAgentsResponseAgentsItemToolRules,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItem,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemChildOutputMapping,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemChildOutputMappingDefaultChild,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemChildOutputMappingDefaultChildItem,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemChildOutputMappingRequireOutputMapping,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemChildOutputMappingType,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemChildren,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemChildrenType,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItem,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildOutputMapping,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildOutputMappingDefaultChild,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildOutputMappingDefaultChildItem,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildOutputMappingRequireOutputMapping,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildOutputMappingType,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildren,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildrenType,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemOne,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemOneType,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemToolName,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemToolNameType,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemTwo,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemTwoType,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemOne,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemOneType,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemToolName,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemToolNameType,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemTwo,
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemTwoType,
    TemplatesCreateAgentsResponseAgentsItemToolsItem,
    TemplatesCreateAgentsResponseAgentsItemToolsItemCreatedById,
    TemplatesCreateAgentsResponseAgentsItemToolsItemCreatedByIdItem,
    TemplatesCreateAgentsResponseAgentsItemToolsItemDescription,
    TemplatesCreateAgentsResponseAgentsItemToolsItemDescriptionItem,
    TemplatesCreateAgentsResponseAgentsItemToolsItemJsonSchema,
    TemplatesCreateAgentsResponseAgentsItemToolsItemJsonSchemaItem,
    TemplatesCreateAgentsResponseAgentsItemToolsItemLastUpdatedById,
    TemplatesCreateAgentsResponseAgentsItemToolsItemLastUpdatedByIdItem,
    TemplatesCreateAgentsResponseAgentsItemToolsItemName,
    TemplatesCreateAgentsResponseAgentsItemToolsItemNameItem,
    TemplatesCreateAgentsResponseAgentsItemToolsItemOrganizationId,
    TemplatesCreateAgentsResponseAgentsItemToolsItemOrganizationIdItem,
    TemplatesCreateAgentsResponseAgentsItemToolsItemSourceCode,
    TemplatesCreateAgentsResponseAgentsItemToolsItemSourceCodeItem,
    TemplatesCreateAgentsResponseAgentsItemToolsItemSourceType,
    TemplatesCreateAgentsResponseAgentsItemToolsItemSourceTypeItem,
    TemplatesCreateAgentsResponseAgentsItemToolsItemToolType,
    TemplatesCreateAgentsResponseAgentsItemUpdatedAt,
    TemplatesCreateAgentsResponseAgentsItemUpdatedAtItem,
)

__all__ = [
    "TemplatesCreateAgentsResponse",
    "TemplatesCreateAgentsResponseAgentsItem",
    "TemplatesCreateAgentsResponseAgentsItemAgentType",
    "TemplatesCreateAgentsResponseAgentsItemBaseTemplateId",
    "TemplatesCreateAgentsResponseAgentsItemBaseTemplateIdItem",
    "TemplatesCreateAgentsResponseAgentsItemCreatedAt",
    "TemplatesCreateAgentsResponseAgentsItemCreatedAtItem",
    "TemplatesCreateAgentsResponseAgentsItemCreatedById",
    "TemplatesCreateAgentsResponseAgentsItemCreatedByIdItem",
    "TemplatesCreateAgentsResponseAgentsItemDescription",
    "TemplatesCreateAgentsResponseAgentsItemDescriptionItem",
    "TemplatesCreateAgentsResponseAgentsItemEmbeddingConfig",
    "TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigAzureDeployment",
    "TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigAzureDeploymentItem",
    "TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigAzureEndpoint",
    "TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigAzureEndpointItem",
    "TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigAzureVersion",
    "TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigAzureVersionItem",
    "TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigEmbeddingChunkSize",
    "TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigEmbeddingChunkSizeItem",
    "TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigEmbeddingEndpoint",
    "TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigEmbeddingEndpointItem",
    "TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigEmbeddingEndpointType",
    "TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigHandle",
    "TemplatesCreateAgentsResponseAgentsItemEmbeddingConfigHandleItem",
    "TemplatesCreateAgentsResponseAgentsItemIdentityIds",
    "TemplatesCreateAgentsResponseAgentsItemLastUpdatedById",
    "TemplatesCreateAgentsResponseAgentsItemLastUpdatedByIdItem",
    "TemplatesCreateAgentsResponseAgentsItemLlmConfig",
    "TemplatesCreateAgentsResponseAgentsItemLlmConfigHandle",
    "TemplatesCreateAgentsResponseAgentsItemLlmConfigHandleItem",
    "TemplatesCreateAgentsResponseAgentsItemLlmConfigMaxTokens",
    "TemplatesCreateAgentsResponseAgentsItemLlmConfigMaxTokensItem",
    "TemplatesCreateAgentsResponseAgentsItemLlmConfigModelEndpoint",
    "TemplatesCreateAgentsResponseAgentsItemLlmConfigModelEndpointItem",
    "TemplatesCreateAgentsResponseAgentsItemLlmConfigModelEndpointType",
    "TemplatesCreateAgentsResponseAgentsItemLlmConfigModelWrapper",
    "TemplatesCreateAgentsResponseAgentsItemLlmConfigModelWrapperItem",
    "TemplatesCreateAgentsResponseAgentsItemLlmConfigPutInnerThoughtsInKwargs",
    "TemplatesCreateAgentsResponseAgentsItemLlmConfigPutInnerThoughtsInKwargsItem",
    "TemplatesCreateAgentsResponseAgentsItemLlmConfigTemperature",
    "TemplatesCreateAgentsResponseAgentsItemMemory",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItem",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemCreatedById",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemCreatedByIdItem",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemDescription",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemDescriptionItem",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemId",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemIsTemplate",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemLabel",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemLabelItem",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemLastUpdatedById",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemLastUpdatedByIdItem",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemLimit",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemMetadata",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemMetadataItem",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemName",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemNameItem",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemOrganizationId",
    "TemplatesCreateAgentsResponseAgentsItemMemoryBlocksItemOrganizationIdItem",
    "TemplatesCreateAgentsResponseAgentsItemMemoryPromptTemplate",
    "TemplatesCreateAgentsResponseAgentsItemMessageBufferAutoclear",
    "TemplatesCreateAgentsResponseAgentsItemMessageIds",
    "TemplatesCreateAgentsResponseAgentsItemMessageIdsItem",
    "TemplatesCreateAgentsResponseAgentsItemMetadata",
    "TemplatesCreateAgentsResponseAgentsItemMetadataItem",
    "TemplatesCreateAgentsResponseAgentsItemOrganizationId",
    "TemplatesCreateAgentsResponseAgentsItemOrganizationIdItem",
    "TemplatesCreateAgentsResponseAgentsItemProjectId",
    "TemplatesCreateAgentsResponseAgentsItemProjectIdItem",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItem",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemCreatedAt",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemCreatedAtItem",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemCreatedById",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemCreatedByIdItem",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemDescription",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemDescriptionItem",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfig",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigAzureDeployment",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigAzureDeploymentItem",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigAzureEndpoint",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigAzureEndpointItem",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigAzureVersion",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigAzureVersionItem",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigEmbeddingChunkSize",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigEmbeddingChunkSizeItem",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigEmbeddingEndpoint",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigEmbeddingEndpointItem",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigEmbeddingEndpointType",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigHandle",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemEmbeddingConfigHandleItem",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemId",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemLastUpdatedById",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemLastUpdatedByIdItem",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemMetadata",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemMetadataItem",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemOrganizationId",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemOrganizationIdItem",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemUpdatedAt",
    "TemplatesCreateAgentsResponseAgentsItemSourcesItemUpdatedAtItem",
    "TemplatesCreateAgentsResponseAgentsItemTemplateId",
    "TemplatesCreateAgentsResponseAgentsItemTemplateIdItem",
    "TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariables",
    "TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItem",
    "TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemCreatedAt",
    "TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemCreatedAtItem",
    "TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemCreatedById",
    "TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemCreatedByIdItem",
    "TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemDescription",
    "TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemDescriptionItem",
    "TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemId",
    "TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemLastUpdatedById",
    "TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemLastUpdatedByIdItem",
    "TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemOrganizationId",
    "TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemOrganizationIdItem",
    "TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemUpdatedAt",
    "TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemUpdatedAtItem",
    "TemplatesCreateAgentsResponseAgentsItemToolRules",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItem",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemChildOutputMapping",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemChildOutputMappingDefaultChild",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemChildOutputMappingDefaultChildItem",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemChildOutputMappingRequireOutputMapping",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemChildOutputMappingType",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemChildren",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemChildrenType",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemItem",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildOutputMapping",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildOutputMappingDefaultChild",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildOutputMappingDefaultChildItem",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildOutputMappingRequireOutputMapping",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildOutputMappingType",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildren",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemChildrenType",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemOne",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemOneType",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemToolName",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemToolNameType",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemTwo",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemTwoType",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemOne",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemOneType",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemToolName",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemToolNameType",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemTwo",
    "TemplatesCreateAgentsResponseAgentsItemToolRulesItemTwoType",
    "TemplatesCreateAgentsResponseAgentsItemToolsItem",
    "TemplatesCreateAgentsResponseAgentsItemToolsItemCreatedById",
    "TemplatesCreateAgentsResponseAgentsItemToolsItemCreatedByIdItem",
    "TemplatesCreateAgentsResponseAgentsItemToolsItemDescription",
    "TemplatesCreateAgentsResponseAgentsItemToolsItemDescriptionItem",
    "TemplatesCreateAgentsResponseAgentsItemToolsItemJsonSchema",
    "TemplatesCreateAgentsResponseAgentsItemToolsItemJsonSchemaItem",
    "TemplatesCreateAgentsResponseAgentsItemToolsItemLastUpdatedById",
    "TemplatesCreateAgentsResponseAgentsItemToolsItemLastUpdatedByIdItem",
    "TemplatesCreateAgentsResponseAgentsItemToolsItemName",
    "TemplatesCreateAgentsResponseAgentsItemToolsItemNameItem",
    "TemplatesCreateAgentsResponseAgentsItemToolsItemOrganizationId",
    "TemplatesCreateAgentsResponseAgentsItemToolsItemOrganizationIdItem",
    "TemplatesCreateAgentsResponseAgentsItemToolsItemSourceCode",
    "TemplatesCreateAgentsResponseAgentsItemToolsItemSourceCodeItem",
    "TemplatesCreateAgentsResponseAgentsItemToolsItemSourceType",
    "TemplatesCreateAgentsResponseAgentsItemToolsItemSourceTypeItem",
    "TemplatesCreateAgentsResponseAgentsItemToolsItemToolType",
    "TemplatesCreateAgentsResponseAgentsItemUpdatedAt",
    "TemplatesCreateAgentsResponseAgentsItemUpdatedAtItem",
]



# This file was auto-generated by Fern from our API Definition.

import typing

TemplatesCreateAgentsResponseAgentsItemLlmConfigPutInnerThoughtsInKwargsItem = typing.Union[bool, typing.Optional[str]]



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
from .templates_create_agents_response_agents_item_tool_rules_item_item_one_type import (
    TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemOneType,
)
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemOne(UncheckedBaseModel):
    tool_name: str
    type: typing.Optional[TemplatesCreateAgentsResponseAgentsItemToolRulesItemItemOneType] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from .templates_create_agents_response_agents_item_tool_exec_environment_variables_item_created_at_item import (
    TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemCreatedAtItem,
)

TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemCreatedAt = typing.Union[
    str,
    typing.Optional[str],
    typing.List[typing.Optional[TemplatesCreateAgentsResponseAgentsItemToolExecEnvironmentVariablesItemCreatedAtItem]],
    typing.Optional[typing.Any],
]
